# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import azure.cli.command_modules.role._help  # pylint: disable=unused-import
from azure.cli.core import AzCommandsLoader
from azure.cli.core.profiles import ResourceType
from ._client_factory import _graph_client_factory as graph_client_factory
from .msgrpah import GraphError


class RoleCommandsLoader(AzCommandsLoader):

    def __init__(self, cli_ctx=None):
        from azure.cli.core.commands import CliCommandType
        role_custom = CliCommandType(operations_tmpl='azure.cli.command_modules.role.custom#{}')
        super(RoleCommandsLoader, self).__init__(cli_ctx=cli_ctx,
                                                 resource_type=ResourceType.MGMT_AUTHORIZATION,
                                                 operation_group='role_assignments',
                                                 custom_command_type=role_custom)

    def load_command_table(self, args):
        if args and args[0] in ('role', 'ad'):
            from knack.log import get_logger
            logger = get_logger(__name__)
            logger.warning("This command or command group has been migrated to Microsoft Graph API. "
                           "Please carefully review all breaking changes introduced during this migration: "
                           "https://docs.microsoft.com/cli/azure/microsoft-graph-migration")
        from azure.cli.command_modules.role.commands import load_command_table
        load_command_table(self, args)
        return self.command_table

    def load_arguments(self, command):
        from azure.cli.command_modules.role._params import load_arguments
        load_arguments(self, command)


COMMAND_LOADER_CLS = RoleCommandsLoader


__all__ = [
    # Public msgraph.GraphClient factory that should be used by other modules
    "graph_client_factory",
    # Public Exception that is raised by msgraph.GraphClient
    "GraphError"
]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.mgmt.reservations.models import (Patch,
                                            PurchaseRequest,
                                            SplitRequest,
                                            MergeRequest,
                                            SkuName,
                                            PurchaseRequestPropertiesReservedResourceProperties)


def cli_reservation_update_reservation(client, reservation_order_id, reservation_id,
                                       applied_scope_type, applied_scopes=None,
                                       instance_flexibility=None):
    if applied_scopes:
        patch = Patch(applied_scope_type=applied_scope_type,
                      applied_scopes=[applied_scopes],
                      instance_flexibility=instance_flexibility)
    else:
        patch = Patch(applied_scope_type=applied_scope_type,
                      instance_flexibility=instance_flexibility)
    return client.begin_update(reservation_order_id, reservation_id, patch)


def create_resource_id(reservation_order_id, reservation_id):
    template = '/providers/Microsoft.Capacity/reservationOrders/{0}/reservations/{1}'
    return template.format(reservation_order_id, reservation_id)


def cli_reservation_split_reservation(client, reservation_order_id,
                                      reservation_id, quantity_1, quantity_2):
    reservationToSplit = create_resource_id(
        reservation_order_id, reservation_id)
    body = SplitRequest(
        quantities=[quantity_1, quantity_2], reservation_id=reservationToSplit)
    return client.begin_split(reservation_order_id, body)


def cli_reservation_merge_reservation(client, reservation_order_id,
                                      reservation_id_1, reservation_id_2):
    body = MergeRequest(sources=[create_resource_id(reservation_order_id, reservation_id_1),
                                 create_resource_id(reservation_order_id, reservation_id_2)])
    return client.begin_merge(reservation_order_id, body)


def cli_calculate(client, sku, reserved_resource_type, billing_scope_id, term,
                  quantity, applied_scope_type, display_name, applied_scope=None,
                  renew=False, instance_flexibility=None, location=None, billing_plan=None):
    sku_name = SkuName(name=sku)
    if applied_scope:
        applied_scopes = [applied_scope]
    else:
        applied_scopes = None
    properties = PurchaseRequestPropertiesReservedResourceProperties(
        instance_flexibility=instance_flexibility)
    body = PurchaseRequest(sku=sku_name, location=location, reserved_resource_type=reserved_resource_type,
                           billing_scope_id=billing_scope_id, term=term, quantity=quantity,
                           display_name=display_name,
                           applied_scope_type=applied_scope_type,
                           applied_scopes=applied_scopes, billing_plan=billing_plan,
                           renew=renew, reserved_resource_properties=properties)
    return client.calculate(body)


def cli_purchase(client, reservation_order_id, sku, reserved_resource_type, billing_scope_id, term,
                 quantity, applied_scope_type, display_name, applied_scope=None,
                 renew=False, instance_flexibility=None, location=None, billing_plan=None):
    sku_name = SkuName(name=sku)
    if applied_scope:
        applied_scopes = [applied_scope]
    else:
        applied_scopes = None
    properties = PurchaseRequestPropertiesReservedResourceProperties(
        instance_flexibility=instance_flexibility)
    body = PurchaseRequest(sku=sku_name, location=location, reserved_resource_type=reserved_resource_type,
                           billing_scope_id=billing_scope_id, term=term, quantity=quantity, display_name=display_name,
                           applied_scope_type=applied_scope_type, applied_scopes=applied_scopes,
                           billing_plan=billing_plan,
                           renew=renew, reserved_resource_properties=properties)
    return client.begin_purchase(reservation_order_id, body)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
from azure.cli.command_modules.monitor._client_factory import _log_analytics_client_factory
from azure.cli.core.commands.transform import _parse_id
from azure.mgmt.loganalytics.models import WorkspaceSkuNameEnum, Workspace, WorkspaceSku, WorkspaceCapping, Table,\
    Schema, Column, SearchResults, RestoredLogs, ColumnTypeEnum
from azure.cli.core.util import sdk_no_wait
from azure.cli.core.azclierror import ArgumentUsageError, InvalidArgumentValueError, RequiredArgumentMissingError
from knack.util import CLIError


def create_log_analytics_workspace(client, resource_group_name, workspace_name, location=None, tags=None,
                                   sku=WorkspaceSkuNameEnum.per_gb2018.value, capacity_reservation_level=None,
                                   retention_time=None, daily_quota_gb=None,
                                   public_network_access_for_query=None, public_network_access_for_ingestion=None,
                                   no_wait=False):
    if sku.lower() == WorkspaceSkuNameEnum.capacity_reservation.value.lower() and capacity_reservation_level is None:
        raise CLIError('--capacity-reservation-level must be set when sku is CapacityReservation.')
    if sku.lower() != WorkspaceSkuNameEnum.capacity_reservation.value.lower() and capacity_reservation_level:
        raise CLIError('--capacity-reservation-level can be set only when sku is CapacityReservation.')

    workspace_client = client
    sku = WorkspaceSku(name=sku, capacity_reservation_level=capacity_reservation_level)
    workspace_capping = WorkspaceCapping(daily_quota_gb=daily_quota_gb)
    workspace_instance = Workspace(location=location,
                                   tags=tags,
                                   sku=sku,
                                   workspace_capping=workspace_capping,
                                   retention_in_days=retention_time,
                                   public_network_access_for_query=public_network_access_for_query,
                                   public_network_access_for_ingestion=public_network_access_for_ingestion)
    return sdk_no_wait(no_wait, workspace_client.begin_create_or_update, resource_group_name,
                       workspace_name, workspace_instance)


def update_log_analytics_workspace(instance, tags=None, capacity_reservation_level=None,
                                   retention_time=None, daily_quota_gb=None,
                                   public_network_access_for_query=None, public_network_access_for_ingestion=None,
                                   default_data_collection_rule_resource_id=None):
    if tags is not None:
        instance.tags = tags
    if capacity_reservation_level is not None:
        if instance.sku.name.lower() != WorkspaceSkuNameEnum.capacity_reservation.value.lower():
            raise CLIError('--capacity-reservation-level can be set only when sku is CapacityReservation.')
        instance.sku.capacity_reservation_level = capacity_reservation_level
    if retention_time is not None:
        instance.retention_in_days = retention_time
    if daily_quota_gb is not None:
        instance.workspace_capping.daily_quota_gb = daily_quota_gb
    if public_network_access_for_query is not None:
        instance.public_network_access_for_query = public_network_access_for_query
    if public_network_access_for_ingestion is not None:
        instance.public_network_access_for_ingestion = public_network_access_for_ingestion
    if default_data_collection_rule_resource_id is not None:
        instance.default_data_collection_rule_resource_id = default_data_collection_rule_resource_id
    return instance


def list_log_analytics_workspace(client, resource_group_name=None):
    workspace_client = client
    if resource_group_name:
        return workspace_client.list_by_resource_group(resource_group_name)
    return workspace_client.list()


def list_deleted_log_analytics_workspaces(client, resource_group_name=None):
    if resource_group_name:
        return client.list_by_resource_group(resource_group_name)
    return client.list()


def recover_log_analytics_workspace(cmd, workspace_name, resource_group_name=None, no_wait=False):
    deleted_workspaces_client = _log_analytics_client_factory(cmd.cli_ctx).deleted_workspaces
    workspace_client = _log_analytics_client_factory(cmd.cli_ctx).workspaces
    deleted_workspaces = list(list_deleted_log_analytics_workspaces(deleted_workspaces_client, resource_group_name))
    for deleted_workspace in deleted_workspaces:
        if deleted_workspace.name.lower() == workspace_name.lower():
            workspace_instance = Workspace(location=deleted_workspace.location)

            return sdk_no_wait(no_wait, workspace_client.begin_create_or_update,
                               _parse_id(deleted_workspace.id)['resource-group'],
                               workspace_name, workspace_instance)

    raise CLIError('{} is not a deleted workspace and you can only recover a deleted workspace within 14 days.'
                   .format(workspace_name))


def _format_tags(tags):
    from azure.mgmt.loganalytics.models import Tag
    if tags:
        tags = [Tag(name=key, value=value) for key, value in tags.items()]
    return tags


def create_log_analytics_workspace_saved_search(client, workspace_name, resource_group_name, saved_search_id,
                                                category, display_name, saved_query,
                                                function_alias=None, function_parameters=None,
                                                tags=None):
    from azure.mgmt.loganalytics.models import SavedSearch
    saved_search = SavedSearch(category=category,
                               display_name=display_name,
                               query=saved_query,
                               function_alias=function_alias,
                               function_parameters=function_parameters,
                               tags=_format_tags(tags))
    return client.create_or_update(resource_group_name=resource_group_name,
                                   workspace_name=workspace_name,
                                   saved_search_id=saved_search_id,
                                   parameters=saved_search)


def update_log_analytics_workspace_saved_search(cmd, instance, category=None, display_name=None, saved_query=None,
                                                function_alias=None, function_parameters=None,
                                                tags=None):
    with cmd.update_context(instance) as c:
        c.set_param('category', category)
        c.set_param('display_name', display_name)
        c.set_param('query', saved_query)
        c.set_param('function_alias', function_alias)
        c.set_param('function_parameters', function_parameters)
        c.set_param('tags', _format_tags(tags))
    return instance


def create_log_analytics_workspace_data_exports(client, workspace_name, resource_group_name, data_export_name,
                                                destination, data_export_type, table_names,
                                                event_hub_name=None, enable=None):
    from azure.mgmt.loganalytics.models import DataExport
    data_export = DataExport(resource_id=destination,
                             data_export_type=data_export_type,
                             table_names=table_names,
                             event_hub_name=event_hub_name,
                             enable=enable)
    return client.create_or_update(resource_group_name, workspace_name, data_export_name, data_export)


def update_log_analytics_workspace_data_exports(cmd, instance, table_names, destination=None, data_export_type=None,
                                                event_hub_name=None, enable=None):
    with cmd.update_context(instance) as c:
        c.set_param('resource_id', destination)
        c.set_param('data_export_type', data_export_type)
        c.set_param('table_names', table_names)
        c.set_param('event_hub_name', event_hub_name)
        c.set_param('enable', enable)
    return instance


# pylint:disable=too-many-locals dangerous-default-value
def create_log_analytics_workspace_table(client, resource_group_name, workspace_name, table_name, columns=[],
                                         retention_in_days=None, total_retention_in_days=None, plan=None,
                                         description=None, no_wait=False):
    if retention_in_days and total_retention_in_days:
        if total_retention_in_days < retention_in_days:
            InvalidArgumentValueError('InvalidArgumentValueError: The specified value of --retention-time'
                                      ' should be less than --total-retention-time')
    columns_list = [] if columns else None
    for col in columns:
        if '=' in col:
            n, t = col.split('=', 1)
            if t.lower() not in [x.value.lower() for x in ColumnTypeEnum]:
                raise InvalidArgumentValueError('InvalidArgumentValueError: "{}" is not a valid value for type_name. '
                                                'Allowed values: string, int, long, real, boolean, dateTime, guid, '
                                                'dynamic".'.format(t))
        else:
            raise ArgumentUsageError('Usage error: --columns should be provided in colunm_name=colunm_type format')
        columns_list.append(Column(name=n, type=t))
    schema = None
    if columns or description is not None:
        if not columns:
            raise RequiredArgumentMissingError('Usage error: When using --description, --columns must be provided')
        schema = Schema(name=table_name, columns=columns_list, description=description)
    table = Table(retention_in_days=retention_in_days,
                  total_retention_in_days=total_retention_in_days,
                  schema=schema,
                  plan=plan
                  )
    return sdk_no_wait(no_wait, client.begin_create_or_update, resource_group_name,
                       workspace_name, table_name, table)


def create_log_analytics_workspace_table_search_job(client, resource_group_name, workspace_name, table_name,
                                                    search_query, start_search_time, end_search_time,
                                                    retention_in_days=None, total_retention_in_days=None, limit=None,
                                                    no_wait=False):
    search_results = SearchResults(query=search_query, limit=limit, start_search_time=start_search_time,
                                   end_search_time=end_search_time)
    table = Table(retention_in_days=retention_in_days,
                  total_retention_in_days=total_retention_in_days,
                  search_results=search_results,
                  )
    return sdk_no_wait(no_wait, client.begin_create_or_update, resource_group_name,
                       workspace_name, table_name, table)


def create_log_analytics_workspace_table_restore(client, resource_group_name, workspace_name, table_name,
                                                 start_restore_time, end_restore_time, restore_source_table,
                                                 no_wait=False):
    restored_logs = RestoredLogs(start_restore_time=start_restore_time,
                                 end_restore_time=end_restore_time,
                                 source_table=restore_source_table)
    table = Table(restored_logs=restored_logs)
    return sdk_no_wait(no_wait, client.begin_create_or_update, resource_group_name,
                       workspace_name, table_name, table)


def update_log_analytics_workspace_table(client, resource_group_name, workspace_name, table_name, columns=[],
                                         retention_in_days=None, total_retention_in_days=None, plan=None,
                                         description=None, no_wait=False):
    columns_list = [] if columns else None
    for col in columns:
        if '=' in col:
            n, t = col.split('=', 1)
            if t.lower() not in [x.value.lower() for x in ColumnTypeEnum]:
                raise InvalidArgumentValueError('InvalidArgumentValueError: "{}" is not a valid value for type_name. '
                                                'Allowed values: string, int, long, real, boolean, dateTime, guid, '
                                                'dynamic".'.format(t))
        else:
            raise ArgumentUsageError('Usage error: --columns should be provided in colunm_name=colunm_type format')
        columns_list.append(Column(name=n, type=t))
    schema = None
    if columns or description is not None:
        schema = Schema(name=table_name, columns=columns_list, description=description)
    table = Table(retention_in_days=retention_in_days,
                  total_retention_in_days=total_retention_in_days,
                  schema=schema,
                  plan=plan
                  )
    return sdk_no_wait(no_wait, client.begin_update, resource_group_name, workspace_name, table_name, table)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def get_sqlvirtualmachine_management_client(cli_ctx):
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    from azure.mgmt.sqlvirtualmachine import SqlVirtualMachineManagementClient

    # Normal production scenario.
    return get_mgmt_service_client(cli_ctx, SqlVirtualMachineManagementClient)


def get_sqlvirtualmachine_availability_group_listeners_operations(cli_ctx, _):
    return get_sqlvirtualmachine_management_client(cli_ctx).availability_group_listeners


def get_sqlvirtualmachine_sql_virtual_machine_groups_operations(cli_ctx, _):
    return get_sqlvirtualmachine_management_client(cli_ctx).sql_virtual_machine_groups


def get_sqlvirtualmachine_sql_virtual_machines_operations(cli_ctx, _):
    return get_sqlvirtualmachine_management_client(cli_ctx).sql_virtual_machines


def get_sqlvirtualmachine_operations(cli_ctx, _):
    return get_sqlvirtualmachine_management_client(cli_ctx).operations



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# The MIT License (MIT)

# Copyright (c) 2016 Blockstack

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# pylint: skip-file

import copy


def process_soa(io, data, name, print_name=False):
    """
    Replace {SOA} in template with a set of serialized SOA records
    """
    indent = ' ' * len('{} {} IN SOA '.format(name, data['ttl']))
    print('{} {} IN SOA {} {} ('.format(name, data['ttl'], data['mname'], data['rname']), file=io)
    for item in ['serial', 'refresh', 'retry', 'expire', 'minimum']:
        print('{}{} ; {}'.format(indent, data[item], item), file=io)
    print('{})'.format(indent), file=io)


def _quote_field(data, field):
    """
    Quote a field in a list of DNS records.
    Return the new data records.
    """
    if data is None:
        return None

    # embedded quotes require escaping - but only if not escaped already
    # note that semi-colons do not need escaping here since we are putting it
    # inside of a quoted string
    fieldBuf = ""
    escape = False
    for c in data[field]:
        if c == '"':
            fieldBuf += '\\"'
            escape = False
        elif c == '\\':
            if escape:
                fieldBuf += '\\\\'
                escape = False
            else:
                escape = True
        else:
            if escape:
                fieldBuf += '\\'
            fieldBuf += c
            escape = False

    data[field] = '"%s"' % fieldBuf

    return data


def process_rr(io, data, record_type, record_keys, name, print_name):
    """ Print out single line record entries """
    if data is None:
        return

    if isinstance(record_keys, str):
        record_keys = [record_keys]
    elif not isinstance(record_keys, list):
        raise ValueError('record_keys must be a string or list of strings')

    name_display = name if print_name else ' ' * len(name)
    print('{} {} IN {} '.format(name_display, data['ttl'], record_type), end='', file=io)

    for i, key in enumerate(record_keys):
        print(data[key], end='\n' if i == len(record_keys) - 1 else ' ', file=io)


def process_ns(io, data, name, print_name=False):
    process_rr(io, data, 'NS', 'host', name, print_name)


def process_a(io, data, name, print_name=False):
    return process_rr(io, data, 'A', 'ip', name, print_name)


def process_aaaa(io, data, name, print_name=False):
    return process_rr(io, data, 'AAAA', 'ip', name, print_name)


def process_caa(io, data, name, print_name=False):
    process_rr(io, _quote_field(data, 'val'), 'CAA', ['flags', 'tag', 'val'], name, print_name)


def process_cname(io, data, name, print_name=False):
    return process_rr(io, data, 'CNAME', 'alias', name, print_name)


def process_mx(io, data, name, print_name=False):
    return process_rr(io, data, 'MX', ['preference', 'host'], name, print_name)


def process_ptr(io, data, name, print_name=False):
    return process_rr(io, data, 'PTR', 'host', name, print_name)


def process_txt(io, data, name, print_name=False):
    return process_rr(io, _quote_field(data, 'txt'), 'TXT', 'txt', name, print_name)


def process_srv(io, data, name, print_name=False):
    return process_rr(io, data, 'SRV', ['priority', 'weight', 'port', 'target'], name, print_name)



# coding=utf-8
# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.help_files import helps  # pylint: disable=unused-import
# pylint: disable=line-too-long, too-many-lines, anomalous-backslash-in-string

helps['rest'] = """
type: command
short-summary: Invoke a custom request.
long-summary: >
    This command automatically authenticates using the logged-in credential: If Authorization header is not set, it
    attaches header `Authorization: Bearer <token>`, where `<token>` is retrieved from AAD. The target resource of the
    token is derived from --url if --url starts with an endpoint from `az cloud show --query endpoints`. You may also
    use --resource for a custom resource.

    If Content-Type header is not set and --body is a valid JSON string, Content-Type header will default to
    application/json.
examples:
  - name: Get Audit log through Microsoft Graph
    text: >
        az rest --method get --url https://graph.microsoft.com/beta/auditLogs/directoryAudits
  - name: Update a Azure Active Directory Graph User's display name
    text: >
        az rest --method patch --url "https://graph.microsoft.com/v1.0/users/johndoe@azuresdkteam.onmicrosoft.com" --body "{\\"displayName\\": \\"jondoe2\\"}"
  - name: Get a virtual machine
    text: >
        az rest --method get --uri /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachines/{vmName}?api-version=2019-03-01
  - name: Create a public IP address from body.json file
    text: >
        az rest --method put --url https://management.azure.com/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/publicIPAddresses/{publicIpAddressName}?api-version=2019-09-01 --body @body.json
  - name: List the top three resources (Bash)
    text: >
        az rest --method get --url https://management.azure.com/subscriptions/{subscriptionId}/resources?api-version=2019-07-01 --url-parameters \\$top=3
"""

helps['version'] = """
type: command
short-summary: Show the versions of Azure CLI modules and extensions in JSON format by default or format configured by --output
"""

helps['upgrade'] = """
type: command
short-summary: Upgrade Azure CLI and extensions
"""

helps['demo'] = """
type: group
short-summary: Demos for designing, developing and demonstrating Azure CLI.
"""

helps['demo style'] = """
type: command
short-summary: A demo showing supported text styles.
"""

helps['demo secret-store'] = """
type: group
short-summary: A demo showing how to use secret store.
"""

helps['demo secret-store save'] = """
type: command
short-summary: Save custom data to secret store.
examples:
  - name: Save data to secret store.
    text: az demo secret-store save "name=Johann Sebastian Bach" job=musician
"""

helps['demo secret-store load'] = """
type: command
short-summary: Load custom data from secret store.
"""



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


def _auth_client_factory(cli_ctx, scope=None):
    import re
    from azure.cli.core.profiles import ResourceType
    from azure.cli.core.commands.client_factory import get_mgmt_service_client
    subscription_id = None
    if scope:
        matched = re.match('/subscriptions/(?P<subscription>[^/]*)/', scope)
        if matched:
            subscription_id = matched.groupdict()['subscription']
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_AUTHORIZATION, subscription_id=subscription_id)


def _graph_client_factory(cli_ctx, **_):
    from .msgrpah import GraphClient
    client = GraphClient(cli_ctx)
    return client
